/*
 *  (c) 2006 by VWP / Oxy
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 */


package com.vwp.sound.mod.modplay;

import java.util.*;
import java.io.*;

/**
 * A list of module names to be played
 *
 * @author torkjel
 */
public class PlayList implements Iterator {
    private Vector list;

    private int position;

    private String playListName = "no-name";

    /**
     * makes a playlist from a file containing filenames seperated by newlines
     *
     * @param fileName the name of a module
     */
    public PlayList(String fileName) throws IOException {
        this.playListName = fileName;
        LineNumberReader lnr = new LineNumberReader(new FileReader(fileName));
        loadList(lnr);
    }

    /**
     * makes a playlist containing several modules
     *
     * @param modules
     */
    public PlayList(String[] modules) throws IOException {

        StringBuffer sb = new StringBuffer();
        for (int n = 0; n < modules.length; n++) {
            sb.append(modules[n]);
            sb.append("\n");
        }
        LineNumberReader lnr = new LineNumberReader(new StringReader(sb
                .toString()));
        loadList(lnr);

    }

    /**
     * loads a list from a LineNumberReader. Each line contains the name of a module.
     *
     * @param lnr
     * @throws IOException
     */
    private void loadList(LineNumberReader lnr) throws IOException {
        position = -1;
        list = new Vector();
        String line = null;
        while ((line = lnr.readLine()) != null) {
            line = line.trim();
            File file = new File(line);
            if (file.exists() && file.isFile())
                list.add(line);
        }
    }

    public String getPlayListName() {
        return playListName;
    }

    /**
     * @return true if there are more modulenames in the list
     */
    public boolean hasNext() {
        return position + 1 < list.size();
    }

    /**
     * @return the next file name in the playlist.
     */
    public Object next() {
        if (hasNext())
            return list.elementAt(++position);
        else
            throw new NoSuchElementException();
    }

    /**
     * @return the next file name in the playlist.
     */
    public String nextFileName() {
        return (String) next();
    }

    /**
     * removes the current module from the playlist
     */
    public void remove() {
        list.removeElementAt(position);
    }

    /**
     * @return the number of files in the playlist
     */
    public int getNumberOfFileNames() {
        return list.size();
    }

    /**
     * @return a random file name from the playlist. This also changes the
     *         position in the playlist
     */
    public String getRandomFileName() {
        int pos = (int) (Math.random() * list.size());
        position = pos - 1;
        return nextFileName();
    }
}
