/*
 *  (c) 2006 by VWP / Oxy
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 */


package com.vwp.sound.mod.modplay.loader;

import com.vwp.sound.mod.modplay.module.ModuleUnits;

/**
 * Conversions between notes, rates and periods used by .XMs
 * @author torkjel
 */
public class XmUnits implements ModuleUnits {

    private static final String NAME = "xm";

    private static double maxNote;
	private static double minNote;

	/** The maximum note value used by .XMs */
	public final static double MAX_NOTE = 96;
	/** The minimum note value used by .XMs */
	public final static double MIN_NOTE = 0;

	public XmUnits()
	{
        maxNote = MAX_NOTE;
        minNote = MIN_NOTE;
	}

	public double period2rate(double period)
	{
        return 8363 * Math.pow(2, ((6 * 12 * 16 * 4 - period) / (12 * 16 * 4)));
    }
	public double rate2period(double rate)
	{
		return
			- 12 * 16 * 4 * Math.log(rate / 8363) / Math.log(2)
			+ 6 * 12 * 16 * 4;
	}
	public double period2note(double period)
	{
        return (period - 10 * 12 * 16 * 4) / (-16 * 4);
		//	return (period-10*12*16*4-4*16)/(-2*16);
	}
	public double note2period(double note)
	{
		return 10 * 12 * 16 * 4 - note * 16 * 4;
		//	return 10*12*16*4 + 4*16 - 2*16*note;
	}
	public double note2rate(double note)
	{
		return period2rate(note2period(note));
	}
	public double rate2note(double rate)
	{
		return period2note(rate2period(rate));
	}
	public double addPeriod(double note, double period)
	{
		return period2note(note2period(note) + period);
	}
	public double getUpperNoteLimit()
	{
		return maxNote;
	}
	public double getLowerNoteLimit()
	{
		return minNote;
	}

    public String getName() {
        return NAME;
    }

}
