/*
 *  (c) 2006 by VWP / Oxy
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 */


package com.vwp.sound.mod.modplay.module;

/**
 * Different module formats uses different relationships between notes, rates and 
 * periods. They also have different limits on maximum and minimum notes.
 * @author torkjel
 *
 */
public interface ModuleUnits
{
    /**
     * converts a period value to a note value 
     */
    public double period2note(double period);

    /** 
     * converts a note value to a period value 
     */    
    public double note2period(double period);

    /** 
     * converts a note value to a rate value 
     */
    public double note2rate(double note);

    /** 
     * converts a rate value to a note value 
     */
    public double rate2note(double note);
    
    /** 
     * adds a period value to a note value 
     */
    public double addPeriod(double note, double period);
    
    /** 
     * gets the highest note that can be played in a module 
     */
    public double getUpperNoteLimit();
    
    /** 
     * gets the lowest note that can be played in a module 
     */
    public double getLowerNoteLimit();

	String getName();
}
