/*
 *  (c) 2006 by VWP / Oxy
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 */


package com.vwp.sound.mod.modplay.module;

/**
 * A pattern contains a number of tracks, each with the same number of divisions
 * @author torkjel
 */
public class Pattern
{
	private Track[] tracks;
	private int divisions;

	/**
	 * @param tracks
	 * @param divisions
	 */
	public Pattern(Track[] tracks, int divisions)
	{
		this.tracks = tracks;
		this.divisions = divisions;
	}

	/**
*	 * @return the n'th track
	 */
	public Track getTrack(int n)
	{
		if (n >= tracks.length || n < 0)
			return null;
		return tracks[n];
	}

	/**
	 * @return the number of tracks in this pattern
	 */
	public int getTrackCount()
	{
		return tracks.length;
	}

	/**
	 * @return the number of divisions in this pattern
	 */
	public int getDivisions()
	{
		return divisions;
	}

	/**
	 * @return a code similar to the codes used in a tracker, representing the 
	 *  given division of this pattern 
	 */
	public String getCode(int division)
	{
		StringBuffer sb = new StringBuffer();
		for (int m = 0; m < tracks.length; m++)
			sb.append(tracks[m].getInfo(division) + " ");
		return sb.toString();
	}

	/**
	 *  @return information about this pattern in human readable form
  	 */
	public String getInfo()
	{
		StringBuffer sb = new StringBuffer();
		sb.append(
			"#tracks: " + tracks.length + " #divisions:" + divisions + "\n");
		for (int n = 0; n < divisions; n++)
		{
			for (int m = 0; m < tracks.length; m++)
				sb.append(tracks[m].getInfo(n) + " ");
			sb.append("\n");
		}
		sb.append("\n");
		return sb.toString();
	}
}
