/*
 *  (c) 2006 by VWP / Oxy
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 */


package com.vwp.sound.mod.modplay.player;

/**
 * @author torkjel
 * Innterface for low level mixers.
 */
public interface LowLevelMixer
{
	/**
	 * Resamples the data in inBuffer, placing the result in outBuffer.
	 *
	 * @param outBuffer is the output buffer
	 * @param outOffsetH outOffset[0] is where the mixer should start writing in outBuffer
	 * @param outLength s how far the mixer should fill outBuffer whith data
	 * @param inBuffer is a circular buffer containing raw data 
	 * @param inOffsetH (outOffset[0] & (inSize-1)) is where the mixer should start to read data from inBuffer
	 * @param inLength the mixer should never read inBuffer further than (inLength & (inSize-1))
	 * @param inSize is the size of inBuffer
	 * @param grad is the realtionship between the samplig rates of the data in inBuffer an outBuffer
	 * 
	 * It IS legal to read data from inBuffer before inOffset[0] and it IS legal 
	 * to not use all the data in inBuffer. This way higher order interpolating can
	 * be implemented.<p>
	 * On return, outOffsetH[0] must contain the index of the last mixed sample 
	 * pluss one in outBuffer and inOffsetH[0] must contain the last mixed sample
	 * pluss one in inBuffer.
	 */
	public void mix(
		int[] outBuffer,
		int[] outOffsetH,
		int outLength,
		short[] inBuffer,
		double[] inOffsetH,
		int inLength,
		int inSize,
		double grad);
}
