/*
 *  (c) 2006 by VWP / Oxy
 *  (c) by Torkjel Hongve
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 */

package com.vwp.sound.mod.util.io;

import java.io.IOException;
import java.io.Writer;

/**
 * a writer that support indentation
 * @author torkjel
 */
public class IndentingWriter extends Writer {

	private static final String INDENT = "  ";
	private String indent = "";

	private Writer writer;

	public IndentingWriter(Writer writer) {
		super();
		this.writer = writer;
	}

	private char lastChar = ' ';

	public void write(char[] cbuf, int off, int len) throws IOException {
		char c;
		for (int i = off; i < off+len; i++) {
			c = cbuf[i];
			if (lastChar == '\n')
				writer.write(indent);
			writer.write(c);
			lastChar = c;
		}
	}

	public void writeln() throws IOException {
		write("\n");
	}

	public void writeln(String str) throws IOException {
		write(str);
		write("\n");
	}

	public void writeRaw(String data) throws IOException {
		writer.write(data);
	}

	public void flush() throws IOException {
		writer.flush();
	}

	public void close() throws IOException {
		writer.close();
	}

	public void indent() {
		indent += INDENT;
	}

	public void unindent() {
		if (indent.length() >= INDENT.length())
			indent = indent.substring(2);
	}
}
