/*
 *  (c) 2006 by VWP / Oxy
 *  (c) 2004 by Torkjel Hongve
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 */

package com.vwp.sound.mod.util.io;

import java.io.IOException;


/**
 * @author torkjel
 */
public class RandomAccessArray extends AbstractRandomAccess {

    private byte[] data;
    private int pos;

    public RandomAccessArray(byte[] data) {
        this.data = data;
        this.pos = 0;
    }

    public void seek(long pos) throws IOException {
        checkBounds(pos);
        this.pos = (int)pos;
    }

    public void skipBytes(int skip) throws IOException {
        checkBounds(pos+skip);
        pos += skip;
    }

    public long getPosition() {
        return pos;
    }

    private void checkBounds(long newPos) throws IOException {
        if (newPos < 0 || newPos >= data.length)
            throw new IOException("Array index out of bounds: " + newPos);
    }

    public byte readByte() throws IOException {
        int data = read();
        if (data == -1) throw new IOException("End of data");
        return (byte)data;
    }

    public short readShort() throws IOException {
        return (short)((readByte() << 8 ) | readByte());
    }

    public int readInt() throws IOException {
        return
            (readByte() << 24) |
            (readByte() << 16) |
            (readByte() << 8) |
            (readByte() << 0);
    }

    public int readUnsignedByte() throws IOException {
        return read() & 0xff;
    }

    public int read() throws IOException {
        checkBounds(pos);
        return data[pos++] & 0xff;
    }

    public void close() {
    }
}